version 1.0
task compare_md5sum {
	input {
	    Array[String] labels
	    Array[File] files
	    Array[File] ref_files
	}

    command <<<
        python <<CODE    
        from collections import OrderedDict
        import os
        import json
        import hashlib

        def md5sum(filename, blocksize=65536):
            hash = hashlib.md5()
            with open(filename, 'rb') as f:
                for block in iter(lambda: f.read(blocksize), b''):
                    hash.update(block)
            return hash.hexdigest()

        with open('~{write_lines(labels)}','r') as fp:
            labels = fp.read().splitlines()
        with open('~{write_lines(files)}','r') as fp:
            files = fp.read().splitlines()
        with open('~{write_lines(ref_files)}','r') as fp:
            ref_files = fp.read().splitlines()

        result = OrderedDict()
        match = OrderedDict()
        match_overall = True

        result['tasks'] = []
        result['failed_task_labels'] = []
        result['succeeded_task_labels'] = []
        for i, label in enumerate(labels):
            f = files[i]
            ref_f = ref_files[i]
            md5 = md5sum(f)
            ref_md5 = md5sum(ref_f)
            # if text file, read in contents
            if f.endswith('.qc') or f.endswith('.txt') or \
                f.endswith('.log') or f.endswith('.out'):
                with open(f,'r') as fp:
                    contents = fp.read()
                with open(ref_f,'r') as fp:
                    ref_contents = fp.read()
            else:
                contents = ''
                ref_contents = ''
            matched = md5==ref_md5
            result['tasks'].append(OrderedDict([
                ('label', label),
                ('match', matched),
                ('md5sum', md5),
                ('ref_md5sum', ref_md5),
                ('basename', os.path.basename(f)),
                ('ref_basename', os.path.basename(ref_f)),
                ('contents', contents),
                ('ref_contents', ref_contents),
                ]))
            match[label] = matched
            match_overall &= matched
            if matched:
                result['succeeded_task_labels'].append(label)
            else:
                result['failed_task_labels'].append(label)        
        result['match_overall'] = match_overall

        with open('result.json','w') as fp:
            fp.write(json.dumps(result, indent=4))
        match_tmp = []
        for key in match:
            val = match[key]
            match_tmp.append('{}\t{}'.format(key, val))
        with open('match.tsv','w') as fp:
            fp.writelines('\n'.join(match_tmp))
        with open('match_overall.txt','w') as fp:
            fp.write(str(match_overall))
        CODE
    >>>
    output {
        Map[String,String] match = read_map('match.tsv') # key:label, val:match
        Boolean match_overall = read_boolean('match_overall.txt')
        File json = glob('result.json')[0] # details (json file)
        String json_str = read_string('result.json') # details (string)
    }
    runtime {
        cpu : 1
        memory : '4000 MB'
        time : 1
        disks : 'local-disk 50 HDD'
    }
}
